/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hive.sample.uihelper

import android.widget.*
import android.content.*
import android.os.Handler
import android.os.Message
import android.text.method.*
import android.text.Editable
import android.util.AttributeSet


/**
 * This is a TextView that is Editable and by default scrollable, like EditText without a cursor.
 *
 * @author ryuvsken
 */
class LogTextView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyle: Int = android.R.attr.textViewStyle) : TextView(context, attrs, defStyle) {


    /**
     * 기본 스크롤 라인 갯수 반환
     */
    /**
     * 기본 스크롤 라인 갯수 설정
     * @param defaultScrollLine    기본 스크롤 라인 갯수
     */
    var defaultScrollLine = 5


    // 안드로이드의 TextView 는 UI 이벤트에서 scrollTo() 를 하면 메시지 내용이 사라지기 때문에 핸들 이벤트로 돌려야 한다.
    private val scrollHandler = object : Handler() {

        //private static final int REFRESH_INTERVAL = 1000;

        override fun handleMessage(msg: Message) {

            when (msg.what) {

                MOVETO_TOP -> {

                    scrollTo(0, 0)
                }

                MOVETO_UP -> {

                    // int l_nLineCount = getLineCount();
                    val l_nPixelsPerLine = lineHeight
                    //int temp = getScrollY();
                    val scrollLine = msg.arg1
                    val l_nCurrentHeight = scrollY - scrollLine * l_nPixelsPerLine
                    val l_nViewHeight = height

                    // The difference between view height and total text height
                    if (l_nCurrentHeight > l_nViewHeight) {
                        scrollTo(0, l_nCurrentHeight - l_nViewHeight)
                    } else {
                        scrollTo(0, l_nCurrentHeight)
                    }
                }

                MOVETO_DOWN -> {

                    // int l_nLineCount = getLineCount();
                    val l_nPixelsPerLine = lineHeight
                    //int temp = getScrollY();
                    val scrollLine = msg.arg1
                    val l_nCurrentHeight = scrollY + scrollLine * l_nPixelsPerLine
                    val l_nViewHeight = height

                    // The difference between view height and total text height
                    if (l_nCurrentHeight > l_nViewHeight) {
                        scrollTo(0, l_nCurrentHeight - l_nViewHeight)
                    } else {
                        scrollTo(0, l_nCurrentHeight)
                    }
                }

                MOVETO_BOTTOM -> {

                    val l_nLineCount = lineCount
                    val l_nViewHeight = height
                    val l_nPixelsPerLine = lineHeight

                    // The difference between view height and total text height
                    val l_nDifference = l_nLineCount * l_nPixelsPerLine
                    if (l_nDifference > l_nViewHeight) {
                        scrollTo(0, l_nDifference - l_nViewHeight)
                    }
                }
            }

            requestLayout()

            // sendEmptyMessageDelayed(0, REFRESH_INTERVAL);
        }
    }


    override fun getDefaultMovementMethod(): MovementMethod {

        return ScrollingMovementMethod.getInstance()
    }


    override fun getText(): Editable {

        return super.getText() as Editable
    }


    override fun setText(text: CharSequence, type: TextView.BufferType?) {

        super.setText(text, TextView.BufferType.EDITABLE)

        val l_nLineCount = lineCount
        val l_nViewHeight = height
        val l_nPixelsPerLine = lineHeight

        // The difference between view height and total text height
        val l_nDifference = l_nLineCount * l_nPixelsPerLine
        if (l_nDifference > l_nViewHeight) {
            scrollTo(0, l_nDifference - l_nViewHeight)
        }
    }


    /**
     * 로그뷰에 로그 추가
     *
     * @param logString    로그 메시지
     */
    fun append(logString: String) {
        var logString = logString

        if (!logString.endsWith("\n"))
            logString += "\n"

        super.append(logString)

        val l_nLineCount = lineCount
        val l_nViewHeight = height
        val l_nPixelsPerLine = lineHeight

        //The difference between view height and total text height
        val l_nDifference = l_nLineCount * l_nPixelsPerLine
        if (l_nDifference > l_nViewHeight) {
            scrollTo(0, l_nDifference - l_nViewHeight)
        }
    }


    /**
     * 로그뷰의 내용을 모두 지운다.
     */
    fun clear() {
        super.setText("")
        scrollTo(0, 0)
    }


    /**
     * 로그뷰의 커서를 최상위로 이동
     */
    fun moveTop() {

        //this.scrollTo(0, 0);
        //scrollHandler.sendEmptyMessage(1);
        val msg = Message.obtain()
        msg.what = MOVETO_TOP
        scrollHandler.sendMessage(msg)
    }


    /**
     * 로그뷰의 커서를 주어진 값만큼 상위로 이동
     *
     * @param line    상위로 이동한 라인 수
     */
    fun moveUp(line: Int) {
        var line = line

        val msg = Message.obtain()
        msg.what = MOVETO_UP

        if (line <= 0) {
            line = defaultScrollLine
        }
        msg.arg1 = line
        scrollHandler.sendMessage(msg)
    }


    /**
     * 로그뷰의 커서를 주어진 값만큼 하위로 이동
     *
     * @param line    하위로 이동한 라인 수
     */
    fun moveDown(line: Int) {
        var line = line

        val msg = Message.obtain()
        msg.what = MOVETO_DOWN
        if (line <= 0) {
            line = defaultScrollLine
        }
        msg.arg1 = line
        scrollHandler.sendMessage(msg)
    }


    /**
     * 로그뷰의 커서를 최하위로 이동
     */
    fun moveBottom() {

        val msg = Message.obtain()
        msg.what = MOVETO_BOTTOM
        scrollHandler.sendMessage(msg)

    }

    companion object {

        private val MOVETO_TOP = 1
        private val MOVETO_UP = 2
        private val MOVETO_DOWN = 3
        private val MOVETO_BOTTOM = 4
    }
}



